/**
 * Copyright (c) 2024 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef _HEAVY_CONTEXT_WSTD_SMTHR_HPP_
#define _HEAVY_CONTEXT_WSTD_SMTHR_HPP_

// object includes
#include "HeavyContext.hpp"
#include "HvSignalEnvelope.h"
#include "HvControlVar.h"
#include "HvSignalLine.h"
#include "HvControlBinop.h"
#include "HvControlIf.h"
#include "HvMath.h"
#include "HvControlCast.h"
#include "HvSignalVar.h"

class Heavy_WSTD_SMTHR : public HeavyContext {

 public:
  Heavy_WSTD_SMTHR(double sampleRate, int poolKb=10, int inQueueKb=2, int outQueueKb=0);
  ~Heavy_WSTD_SMTHR();

  const char *getName() override { return "WSTD_SMTHR"; }
  int getNumInputChannels() override { return 2; }
  int getNumOutputChannels() override { return 2; }

  int process(float **inputBuffers, float **outputBuffer, int n) override;
  int processInline(float *inputBuffers, float *outputBuffer, int n) override;
  int processInlineInterleaved(float *inputBuffers, float *outputBuffer, int n) override;

  int getParameterInfo(int index, HvParameterInfo *info) override;
  struct Parameter {
    struct In {
      enum ParameterIn : hv_uint32_t {
        LMTR = 0xCB23DC60, // Lmtr
        MIX = 0x97D46AC6, // Mix
        SMTHR = 0xD88E26F9, // Smthr
      };
    };
  };

 private:
  HvTable *getTableForHash(hv_uint32_t tableHash) override;
  void scheduleMessageForReceiver(hv_uint32_t receiverHash, HvMessage *m) override;

  // static sendMessage functions
  static void cBinop_5gl1DN7r_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_wf85TgEZ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_0N4bqCqq_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_Oo14RSyQ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_7q2qLi3g_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_B03g76If_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_mbl2SeTg_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_NsYMNWEL_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_BQf0uN7d_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_vHkwpuDe_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_D7mvGS2l_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_cMkAlzBM_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_bcpR7ZUD_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_OaTeOOv2_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_euYdC4Fd_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_nezXn2ez_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_t89PEmku_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_cNnK4dIb_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_sZiIwQnu_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_MPkg4E1d_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_jCaTJajx_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_YPtcp9is_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_M4s4IxMt_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_IEds3z1S_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_tmq1o5i2_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_vvDe5tUH_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_HSX54Vqv_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_ANfRLEft_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_HEsb1NXM_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_NAETkO9M_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_3kWcN7Nv_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_hqjeVFop_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_iqEduZ5o_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_LNGY0Gcy_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_YA6VfzTr_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_UXSfY2jg_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_TyidPsaJ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_OpUTdWJV_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_5sRLLfj9_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_1vn2mQmo_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_dhBzGYIN_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_pFk9lcH2_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_J8gTOGJe_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_6OIBTjYs_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_ZOb1QE1N_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_N4tItXRo_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_WsyW3kge_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_kNHXlStg_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_he2PYsiJ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_iygs3Vp4_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_g3ykg8on_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_l3fY183s_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cSend_cSzhPJz8_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_Cth2wKuN_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_yVLzQ6Qb_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_4carrwVI_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_j0CyHui1_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_F34kL7ZJ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_AqumPWXc_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_AE8F1BMJ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_LBmdGlId_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_eMi2ctPN_sendMessage(HeavyContextInterface *, int, const HvMessage *);

  // objects
  SignalLine sLine_1iCsLOVp;
  SignalEnvelope sEnv_cNnK4dIb;
  SignalEnvelope sEnv_sZiIwQnu;
  SignalEnvelope sEnv_YA6VfzTr;
  SignalEnvelope sEnv_UXSfY2jg;
  SignalLine sLine_JYFBQOtu;
  SignalLine sLine_UcCU26tO;
  ControlBinop cBinop_5gl1DN7r;
  ControlBinop cBinop_wf85TgEZ;
  ControlBinop cBinop_0N4bqCqq;
  ControlBinop cBinop_Oo14RSyQ;
  ControlBinop cBinop_mbl2SeTg;
  ControlBinop cBinop_BQf0uN7d;
  ControlBinop cBinop_vHkwpuDe;
  ControlBinop cBinop_D7mvGS2l;
  ControlVar cVar_cMkAlzBM;
  ControlVar cVar_OaTeOOv2;
  SignalVarf sVarf_Tsbp5VE0;
  ControlVar cVar_euYdC4Fd;
  ControlIf cIf_nezXn2ez;
  ControlIf cIf_t89PEmku;
  ControlBinop cBinop_MPkg4E1d;
  ControlBinop cBinop_YPtcp9is;
  ControlBinop cBinop_M4s4IxMt;
  ControlBinop cBinop_vvDe5tUH;
  ControlBinop cBinop_ANfRLEft;
  SignalVarf sVarf_84S0ksAk;
  ControlVar cVar_hqjeVFop;
  ControlIf cIf_iqEduZ5o;
  ControlIf cIf_LNGY0Gcy;
  ControlBinop cBinop_TyidPsaJ;
  ControlBinop cBinop_5sRLLfj9;
  ControlBinop cBinop_1vn2mQmo;
  ControlBinop cBinop_J8gTOGJe;
  ControlBinop cBinop_ZOb1QE1N;
  ControlVar cVar_he2PYsiJ;
  ControlBinop cBinop_iygs3Vp4;
  SignalVarf sVarf_7woKXPY2;
  SignalVarf sVarf_LWx0VPzy;
  SignalVarf sVarf_sdp5wSkI;
  SignalVarf sVarf_aedomag9;
  ControlBinop cBinop_Cth2wKuN;
};

#endif // _HEAVY_CONTEXT_WSTD_SMTHR_HPP_
